// Integer/fractional Base Converter
// Author: Swapnil Jain
// This program converts a number(integer or float or both) in 
//   any base btwn 2 to 16 to its corresponding number in base 10. 

#include <tigcclib.h>
#include <math.h>
typedef unsigned long int bigint;

int getBase(); //forward declaration
void baseXto10(int base); //forward declaration


// Main Function
void _main()
{
  FontSetSys(2); //set large font
	ClrScr(); //clear the screen
	DrawStr(5, 34, "Integer/Fractional", A_NORMAL);
	DrawStr(15, 45, "Base Converter", A_NORMAL);
	FontSetSys(1); //set medium font
	DrawStr(30, 60, "by Swapnil Jain", A_NORMAL);
  ngetchx(); //wait for input 

  int base = 0;
  
	clrscr(); //clear the screen
	FontSetSys(0); //set small font
  
  printf("\nEnter a base:\n");
  base = getBase();
  
  if (base > 16 || base < 2)
  { //if invalid base
    printf("\n\nERROR! Invalid Base.\n");
    printf("\nPRESS ANY KEY TO EXIT ...\n");
    ngetchx();
    return; //terminate prgm
  }

  printf("\n\nEnter an  integer in that base:\n"); 
  baseXto10(base);
    
  printf("\n\nPRESS ANY KEY TO EXIT ...\n");
	ngetchx(); // wait for input before exiting
} //main()


void baseXto10(int base)
{ //this function basically does a lot of things at once
  short key = 0, digit = 0, flag = 0, count = 0; 
  bigint numerator = 0, denominator = 1,
    decNum = 0, overflow = 0, overflow1 = 0, overflow2 = 0;

	
	while((key = ngetchx()) != KEY_ENTER)
	{ //while the key is not "ENTER" i.e "\n"
	
	  if (key >= 48 && key <= 57 && key < (base + 48))
	  { //if a numeric/valid character
	    digit = key - 48; //remove ascii bias
	    printf("%hd", digit); //display the input
	    
	    if (flag == 0)
	    { //if number is an integer
	    decNum = decNum*base + digit; //increase the weight and add
	    }
	    else
	    {
	  	  numerator = numerator*base + digit;
	      denominator = denominator*base;
	    }
	  }
	  else if (key >= 65 && key <= 70 && key < (base + 55) )
	  { //if an alphanumeric/valid character
	    switch(key)
	    {
	  		case 65: digit = 10; printf("A"); break;
	  		case 66: digit = 11; printf("B"); break;
	  		case 67: digit = 12; printf("C"); break;
	  		case 68: digit = 13; printf("D"); break;
	  		case 69: digit = 14; printf("E"); break;	  		
	  		case 70: digit = 15; printf("F"); break;	  		
	  		default: digit = 0; break;
	    }
	    
	    if (flag == 0)
	    { //if number is an integer
	      decNum = decNum*base + digit; //increase the weight and add
	    }
	    else
	    {
	  	  numerator = numerator*base + digit;
	      denominator = denominator*base; 
	    }
	  }
	  else if (key == 46 && flag != 1) 
	  { //if the radix point hasn't been inputed already
	    printf(".");
	    flag = 1;
	  } 
	  
    if (decNum < overflow) //if >= 2^32 - 1  
	  { //in the case of an overflow
	    // for some reason, the decNum doesn't loop back to O
	      // so this is the only possible overflow test
	  	printf("\n\nERROR! OVERFLOW.\n");
	  	return;
	  }
	  overflow = decNum;
	  
	if(overflow1 > denominator || overflow2 > numerator)
	  { //in the case of an overflow
	  	printf("\n\nERROR! OVERFLOW.\n");
	  	return;
	  }
	  overflow1 = denominator; //checking overflow
	  overflow2 = numerator; //checking overflow
	} //end while


  printf("\n\nThe exact base-10 number is:\n");
  printf("%lu", decNum); //print the integer part
    
  if(flag == 1)
  { //if the number is a float
    printf(".");
    digit = 0;
       
    while (numerator != 0 && count != 100)
    { //output the float part
      count++;
  	  numerator = (numerator - digit*denominator)*10;
      digit = numerator/denominator;
      printf("%d", digit);
    }
  }

  if (count == 100)
  { //if a repeating decimal
  	printf(" ...");
  }
  
} //baseXto10()


int getBase()
{ //this function displays and takes the base from the user 
	short key = 0, digit = 0; 
	int base = 0;

	while( (key = ngetchx()) != KEY_ENTER)
	{ //while input is not a linefeed
	  digit = key - 48; //remove the ascii bias
	  
	  if (digit >= 0 && digit <= 9)
	  { //if input is valid
	    printf("%hd", digit);	//display the input
		  base = base*10 + digit; //increase the weight and add	
	  } 
	}
	
  return base;
} //getBase()
